@@ -27,8 +27,8 @@ from .utils import generate_checksum
 logger = logging.getLogger(__name__)
 
 
-# Module cache to reduce initialization costs
-constructed_views_cache = LRUCache(maxsize=100)
+# Module cache to store the found component classes
+views_cache = LRUCache(maxsize=100)
 
 
 class UnicornField:
@@ -115,12 +115,57 @@ def convert_to_snake_case(s: str) -> str:
     return s.replace("-", "_")
 
 
-def convert_to_camel_case(s: str) -> str:
-    # TODO: Better handling of dash/snake->camel-case
+def convert_to_pascal_case(s: str) -> str:
+    # TODO: Better handling of dash/snake->pascal-case
     s = convert_to_snake_case(s)
     return "".join(word.title() for word in s.split("_"))
 
 
+def get_locations(component_name):
+    # TODO: django.conf setting bool that defines whether to look in all installed apps for components
+    locations = []
+
+    if "." in component_name:
+        # Handle component names that specify a folder structure
+        component_name = component_name.replace("/", ".")
+
+        # Handle fully-qualified component names (e.g. `project.unicorn.HelloWorldView`)
+        class_name = component_name.split(".")[-1:][0]
+        module_name = component_name.replace("." + class_name, "")
+        locations.append((class_name, module_name))
+
+        # Assume if it ends with "View", then we don't need to add other
+        if component_name.endswith("View") or component_name.endswith("Component"):
+            return locations
+
+    # Handle component names that specify a folder structure
+    component_name = component_name.replace("/", ".")
+
+    # Use conventions to find the component class
+    class_name = convert_to_pascal_case(component_name)
+
+    if "." in class_name:
+        if class_name.split(".")[-1:]:
+            class_name = class_name.split(".")[-1:][0]
+
+    class_name = f"{class_name}View"
+    module_name = convert_to_snake_case(component_name)
+
+    unicorn_apps = get_setting("APPS", ["unicorn"])
+
+    assert (
+        isinstance(unicorn_apps, list)
+        or isinstance(unicorn_apps, tuple)
+        or isinstance(unicorn_apps, set)
+    ), "APPS is expected to be a list, tuple or set"
+
+    for app in unicorn_apps:
+        app_module_name = f"{app}.components.{module_name}"
+        locations.append((class_name, app_module_name))
+
+    return locations
+
+
 class UnicornTemplateResponse(TemplateResponse):
     def __init__(
         self,
@@ -161,28 +206,33 @@ class UnicornTemplateResponse(TemplateResponse):
         frontend_context_variables_dict = orjson.loads(frontend_context_variables)
         checksum = generate_checksum(orjson.dumps(frontend_context_variables_dict))
 
-        # Handle potential nested component name using dot-notation
-        nested_component_name = self.component.component_name.replace(".", "/")
-
         soup = BeautifulSoup(content, features="html.parser")
         root_element = UnicornTemplateResponse._get_root_element(soup)
         root_element["unicorn:id"] = self.component.component_id
-        root_element["unicorn:name"] = nested_component_name
+        root_element["unicorn:name"] = self.component.component_name
         root_element["unicorn:key"] = self.component.component_key
         root_element["unicorn:checksum"] = checksum
 
         if self.init_js:
-            script_tag = soup.new_tag("script")
             init = {
                 "id": self.component.component_id,
-                "name": nested_component_name,
+                "name": self.component.component_name,
                 "key": self.component.component_key,
                 "data": orjson.loads(frontend_context_variables),
             }
             init = orjson.dumps(init).decode("utf-8")
-            script_tag["type"] = "module"
-            script_tag.string = f"if (typeof Unicorn === 'undefined') {{ console.error('Unicorn is missing. Do you need {{% load unicorn %}} or {{% unicorn-scripts %}}?') }} else {{ Unicorn.componentInit({init}); }}"
-            root_element.insert_after(script_tag)
+            init_script = f"Unicorn.componentInit({init});"
+
+            if self.component.parent:
+                self.component._init_script = init_script
+            else:
+                for child in self.component.children:
+                    init_script = f"{init_script} {child._init_script}"
+
+                script_tag = soup.new_tag("script")
+                script_tag["type"] = "module"
+                script_tag.string = f"if (typeof Unicorn === 'undefined') {{ console.error('Unicorn is missing. Do you need {{% load unicorn %}} or {{% unicorn-scripts %}}?') }} else {{ {init_script} }}"
+                root_element.insert_after(script_tag)
 
         rendered_template = UnicornTemplateResponse._desoupify(soup)
         rendered_template = mark_safe(rendered_template)
@@ -247,6 +297,7 @@ class UnicornView(TemplateView):
         if "parent" in kwargs:
             self.parent = kwargs["parent"]
 
+        self._init_script: str = ""
         self._children_set = False
         self._validate_called = False
         self.errors = {}
@@ -674,7 +725,6 @@ class UnicornView(TemplateView):
         component_name: str,
         component_key: str = "",
         parent: "UnicornView" = None,
-        use_cache=True,
         request: HttpRequest = None,
         kwargs: Dict[str, Any] = {},
     ) -> "UnicornView":
@@ -688,7 +738,6 @@ class UnicornView(TemplateView):
             param component_key: Key of the component to allow multiple components of the same name
                 to be differentiated. Optional.
             param parent: The parent component of the current component.
-            param use_cache: Get component from cache or force construction of component. Defaults to `True`.
             param kwargs: Keyword arguments for the component passed in from the template. Defaults to `{}`.
         
         Returns:
@@ -698,17 +747,6 @@ class UnicornView(TemplateView):
         assert component_id, "Component id is required"
         assert component_name, "Component name is required"
 
-        if use_cache:
-            if component_id in constructed_views_cache:
-                component = constructed_views_cache[component_id]
-                component.setup(request)
-                component._validate_called = False
-                logger.debug(f"Retrieve {component_id} from constructed views cache")
-
-                return component
-
-        locations = []
-
         @timed
         def _get_component_class(
             module_name: str, class_name: str
@@ -721,38 +759,53 @@ class UnicornView(TemplateView):
 
             return component_class
 
-        if "." in component_name:
-            # Handle fully-qualified component names (e.g. `project.unicorn.HelloWorldView`)
-            class_name = component_name.split(".")[-1:][0]
-            module_name = component_name.replace("." + class_name, "")
-            locations.append((class_name, module_name))
+        @timed
+        def _construct_component(
+            component_class,
+            component_id,
+            component_name,
+            component_key,
+            parent,
+            request,
+            **kwargs,
+        ):
+            """
+            Constructs a class instance.
+            """
+            component = component_class(
+                component_id=component_id,
+                component_name=component_name,
+                component_key=component_key,
+                parent=parent,
+                request=request,
+                **kwargs,
+            )
 
-        # Handle component names that specify a folder structure
-        component_name = component_name.replace("/", ".")
+            component.children = []
+            component._children_set = False
 
-        # Use conventions to find the component class
-        class_name = convert_to_camel_case(component_name)
+            component.mount()
+            component.hydrate()
+            component._validate_called = False
 
-        if "." in class_name:
-            if class_name.split(".")[-1:]:
-                class_name = class_name.split(".")[-1:][0]
+            return component
 
-        class_name = f"{class_name}View"
-        module_name = convert_to_snake_case(component_name)
+        if component_id in views_cache:
+            component_class = views_cache[component_id]
+            component = _construct_component(
+                component_class=component_class,
+                component_id=component_id,
+                component_name=component_name,
+                component_key=component_key,
+                parent=parent,
+                request=request,
+                **kwargs,
+            )
+            logger.debug(f"Retrieve {component_id} from views cache")
 
-        unicorn_apps = get_setting("APPS", ["unicorn"])
+            return component
 
-        assert (
-            isinstance(unicorn_apps, list)
-            or isinstance(unicorn_apps, tuple)
-            or isinstance(unicorn_apps, set)
-        ), "APPS is expected to be a list, tuple or set"
-
-        for app in unicorn_apps:
-            app_module_name = f"{app}.components.{module_name}"
-            locations.append((class_name, app_module_name))
-
-        # TODO: django.conf setting bool that defines whether to look in all installed apps for components
+        locations = get_locations(component_name)
 
         # Store the last exception that got raised while looking for a component in case it is useful context
         last_exception: Union[
@@ -762,7 +815,8 @@ class UnicornView(TemplateView):
         for (class_name, module_name) in locations:
             try:
                 component_class = _get_component_class(module_name, class_name)
-                component = component_class(
+                component = _construct_component(
+                    component_class=component_class,
                     component_id=component_id,
                     component_name=component_name,
                     component_key=component_key,
@@ -770,16 +824,8 @@ class UnicornView(TemplateView):
                     request=request,
                     **kwargs,
                 )
-
-                component.children = []
-                component._children_set = False
-
-                component.mount()
-                component.hydrate()
-                component._validate_called = False
-
-                # Put the component in a "cache" to skip looking for the component on the next request
-                constructed_views_cache[component_id] = component
+                # Put the component's class in a "cache" to skip looking for the component on the next request
+                views_cache[component_id] = component_class
 
                 return component
             except ModuleNotFoundError as e:
