@@ -1,8 +1,24 @@
+import logging
 from functools import lru_cache
 from typing import Any, Dict, List
 
 from django.core.serializers import serialize
-from django.db.models import Model, QuerySet
+from django.db.models import (
+    DateField,
+    DateTimeField,
+    DurationField,
+    Model,
+    QuerySet,
+    TimeField,
+)
+from django.utils.dateparse import (
+    parse_date,
+    parse_datetime,
+    parse_duration,
+    parse_time,
+)
+
+import orjson
 
 
 try:
@@ -10,21 +26,47 @@ try:
 except ImportError:
     PydanticBaseModel = None
 
-import orjson
+
+logger = logging.getLogger(__name__)
 
 
 class JSONDecodeError(Exception):
     pass
 
 
-def _get_model_dict(obj: Any) -> dict:
+def _parse_field_values_from_string(model: Model) -> None:
+    """
+    Convert the model fields' value to match the field type if appropriate.
+
+    This is mostly to deal field string values that will get saved as a date-related field.
+    """
+
+    for field in model._meta.fields:
+        val = getattr(model, field.attname)
+
+        if not isinstance(val, str):
+            continue
+
+        if isinstance(field, DateTimeField):
+            setattr(model, field.attname, parse_datetime(val))
+        elif isinstance(field, TimeField):
+            setattr(model, field.attname, parse_time(val))
+        elif isinstance(field, DateField):
+            setattr(model, field.attname, parse_date(val))
+        elif isinstance(field, DurationField):
+            setattr(model, field.attname, parse_duration(val))
+
+
+def _get_model_dict(model: Model) -> dict:
     """
     Serializes Django models. Uses the built-in Django JSON serializer, but moves the data around to
     remove some unnecessary information and make the structure more compact.
     """
 
-    # Django's serialize always returns an array, so remove the brackets from the string
-    serialized_model = serialize("json", [obj])[1:-1]
+    _parse_field_values_from_string(model)
+
+    # Django's `serialize` method always returns an array, so remove the brackets from the resulting string
+    serialized_model = serialize("json", [model])[1:-1]
     model_json = orjson.loads(serialized_model)
     model_pk = model_json.get("pk")
     model_json = model_json.get("fields")
@@ -45,26 +87,30 @@ def _json_serializer(obj):
     """
     from .components import UnicornView
 
-    if isinstance(obj, UnicornView):
-        return {
-            "name": obj.component_name,
-            "id": obj.component_id,
-            "key": obj.component_key,
-        }
-    elif isinstance(obj, Model):
-        return _get_model_dict(obj)
-    elif isinstance(obj, QuerySet):
-        queryset_json = []
+    try:
+        if isinstance(obj, UnicornView):
+            return {
+                "name": obj.component_name,
+                "id": obj.component_id,
+                "key": obj.component_key,
+            }
+        elif isinstance(obj, Model):
+            return _get_model_dict(obj)
+        elif isinstance(obj, QuerySet):
+            queryset_json = []
 
-        for model in obj:
-            model_json = _get_model_dict(model)
-            queryset_json.append(model_json)
+            for model in obj:
+                model_json = _get_model_dict(model)
+                queryset_json.append(model_json)
 
-        return queryset_json
-    elif PydanticBaseModel and isinstance(obj, PydanticBaseModel):
-        return obj.dict()
-    elif hasattr(obj, "to_json"):
-        return obj.to_json()
+            return queryset_json
+        elif PydanticBaseModel and isinstance(obj, PydanticBaseModel):
+            return obj.dict()
+        elif hasattr(obj, "to_json"):
+            return obj.to_json()
+    except Exception as e:
+        # Log this because the `TypeError` and resulting stacktrace lacks context
+        logger.exception(e)
 
     raise TypeError
 
@@ -122,6 +168,8 @@ def dumps(data: dict, fix_floats: bool = True) -> str:
     Args:
         param fix_floats: Whether any floats should be converted to strings. Defaults to `True`,
             but will be faster without it.
+    
+    Returns a string which deviates from `orjson.dumps`, but seems more useful.
     """
 
     serialized_data = orjson.dumps(data, default=_json_serializer)
@@ -129,7 +177,7 @@ def dumps(data: dict, fix_floats: bool = True) -> str:
     if fix_floats:
         return _dumps(serialized_data)
 
-    return serialized_data
+    return serialized_data.decode("utf-8")
 
 
 def loads(str: str) -> dict:
