@@ -30,6 +30,7 @@ export function send(component, callback) {
     data: component.data,
     checksum: component.checksum,
     actionQueue: component.currentActionQueue,
+    epoch: Date.now(),
   };
 
   const headers = {
@@ -57,6 +58,10 @@ export function send(component, callback) {
         return;
       }
 
+      if (responseJson.queued && responseJson.queued === true) {
+        return;
+      }
+
       if (responseJson.error) {
         if (responseJson.error === "Checksum does not match") {
           // Reset the models if the checksum doesn't match
@@ -98,8 +103,11 @@ export function send(component, callback) {
         element.handleDirty(true);
       });
 
-      // Get the data from the response
-      component.data = responseJson.data || {};
+      // Merge the data from the response into the component's data
+      Object.keys(responseJson.data || {}).forEach((key) => {
+        component.data[key] = responseJson.data[key];
+      });
+
       component.errors = responseJson.errors || {};
       component.return = responseJson.return || {};
 
@@ -128,19 +136,32 @@ export function send(component, callback) {
       }
 
       // Refresh the parent component if there is one
-      if (hasValue(parent) && hasValue(parent.id) && hasValue(parent.dom)) {
+      if (hasValue(parent) && hasValue(parent.id)) {
         const parentComponent = component.getParentComponent(parent.id);
 
         if (parentComponent && parentComponent.id === parent.id) {
-          // TODO: Handle errors?
-          parentComponent.data = parent.data || {};
+          // TODO: Handle parent errors?
+
+          if (hasValue(parent.data)) {
+            parentComponent.data = parent.data;
+          }
+
+          if (parent.dom) {
+            component.morphdom(
+              parentComponent.root,
+              parent.dom,
+              MORPHDOM_OPTIONS
+            );
+          }
+
+          if (parent.checksum) {
+            parentComponent.root.setAttribute(
+              "unicorn:checksum",
+              parent.checksum
+            );
+            parentComponent.refreshChecksum();
+          }
 
-          component.morphdom(
-            parentComponent.root,
-            parent.dom,
-            MORPHDOM_OPTIONS
-          );
-          parentComponent.refreshChecksum();
           parentComponent.refreshEventListeners();
 
           parentComponent.getChildrenComponents().forEach((child) => {
@@ -161,6 +182,14 @@ export function send(component, callback) {
             targetDom = $(`#${partial.id}`, component.root);
           }
 
+          if (!targetDom && component.root.parentElement) {
+            // Go up one parent if the target can't be found
+            targetDom = $(
+              `[unicorn\\:key="${partial.key}"]`,
+              component.root.parentElement
+            );
+          }
+
           if (targetDom) {
             component.morphdom(targetDom, partial.dom, MORPHDOM_OPTIONS);
           }
