@@ -1,7 +1,9 @@
+import copy
 import logging
 from functools import wraps
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, List, Union, get_type_hints
 
+from django.core.cache import caches
 from django.db.models import Model
 from django.http import HttpRequest, JsonResponse
 from django.views.decorators.csrf import csrf_protect
@@ -15,7 +17,8 @@ from .components import UnicornField, UnicornView
 from .decorators import timed
 from .errors import UnicornViewError
 from .message import ComponentRequest, Return
-from .serializer import loads
+from .serializer import dumps, loads
+from .settings import get_cache_alias, get_serial_enabled, get_serial_timeout
 from .utils import generate_checksum
 
 
@@ -35,31 +38,88 @@ def handle_error(view_func):
     return wraps(view_func)(wrapped_view)
 
 
+@timed
+def _is_component_field_model_or_unicorn_field(
+    component_or_field: Union[UnicornView, UnicornField, Model], name: str,
+) -> bool:
+    """
+    Determines whether a component's field is a Django `Model` or `UnicornField` either
+    by checking the field's instance or inspecting the type hints.
+
+    One side-effect is that the field will be instantiated if it is currently `None` and
+    the type hint is available.
+
+    Args:
+        component: `UnicornView` to check.
+        name: Name of the field.
+
+    Returns:
+        Whether the field is a Django `Model` or `UnicornField`.
+    """
+    field = getattr(component_or_field, name)
+
+    if isinstance(field, UnicornField) or isinstance(field, Model):
+        return True
+
+    is_subclass_of_model = False
+    is_subclass_of_unicorn_field = False
+    component_type_hints = {}
+
+    try:
+        component_type_hints = get_type_hints(component_or_field)
+
+        if name in component_type_hints:
+            is_subclass_of_model = issubclass(component_type_hints[name], Model)
+
+            if not is_subclass_of_model:
+                is_subclass_of_unicorn_field = issubclass(
+                    component_type_hints[name], UnicornField
+                )
+
+            # Construct a new class if the field is None and there is a type hint available
+            if field is None:
+                if is_subclass_of_model or is_subclass_of_unicorn_field:
+                    field = component_type_hints[name]()
+                    setattr(component_or_field, name, field)
+    except TypeError as e:
+        pass
+
+    return is_subclass_of_model or is_subclass_of_unicorn_field
+
+
 @timed
 def _set_property_from_data(
-    component_or_field: Union[UnicornView, UnicornField, Model], name: str, value,
+    component_or_field: Union[UnicornView, UnicornField, Model], name: str, value: Any,
 ) -> None:
     """
     Sets properties on the component based on passed-in data.
     """
 
-    if hasattr(component_or_field, name):
+    if not hasattr(component_or_field, name):
+        return
+
+    field = getattr(component_or_field, name)
+    component_field_is_model_or_unicorn_field = _is_component_field_model_or_unicorn_field(
+        component_or_field, name
+    )
+
+    # UnicornField and Models are always a dictionary (can be nested)
+    if component_field_is_model_or_unicorn_field:
+        # Re-get the field since it might have been set in `_is_component_field_model_or_unicorn_field`
         field = getattr(component_or_field, name)
 
-        # UnicornField and Models are always a dictionary (can be nested)
-        if isinstance(field, UnicornField) or isinstance(field, Model):
-            if isinstance(value, dict):
-                for key in value.keys():
-                    key_value = value[key]
-                    _set_property_from_data(field, key, key_value)
-            else:
-                _set_property_from_data(field, field.name, value)
+        if isinstance(value, dict):
+            for key in value.keys():
+                key_value = value[key]
+                _set_property_from_data(field, key, key_value)
         else:
-            if hasattr(component_or_field, "_set_property"):
-                # Can assume that `component_or_field` is a component
-                component_or_field._set_property(name, value)
-            else:
-                setattr(component_or_field, name, value)
+            _set_property_from_data(field, field.name, value)
+    else:
+        if hasattr(component_or_field, "_set_property"):
+            # Can assume that `component_or_field` is a component
+            component_or_field._set_property(name, value)
+        else:
+            setattr(component_or_field, name, value)
 
 
 @timed
@@ -186,40 +246,44 @@ def _call_method_name(
             return func()
 
 
-@timed
-@handle_error
-@csrf_protect
-@require_POST
-def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
+def _process_component_request(
+    request: HttpRequest, component_request: ComponentRequest
+) -> Dict:
     """
-    Endpoint that instantiates the component and does the correct action
-    (set an attribute or call a method) depending on the JSON payload in the body.
-
+    Process a `ComponentRequest` by:
+        1. construct a Component view
+        2. set all of the properties on the view from the data
+        3. execute the type
+            - update the properties based on the payload for "syncInput"
+            - create/update the Django Model based on the payload for "dbInput"
+            - call the method specified for "callMethod"
+        4. validate any fields specified in a Django form
+        5. construct a `dict` that will get returned in a `JsonResponse` later on
+    
     Args:
         param request: HttpRequest for the function-based view.
-        param: component_name: Name of the component, e.g. "hello-world".
+        param: component_request: Component request to process.
     
     Returns:
-        JSON with the following structure:
+        `dict` with the following structure:
         {
             "id": component_id,
             "dom": html,  // re-rendered version of the component after actions in the payload are completed
             "data": {},  // updated data after actions in the payload are completed
+            "errors": {},  // form validation errors
+            "return": {}, // optional return value from an executed action
+            "parent": {},  // optional representation of the parent component
         }
     """
-
-    assert component_name, "Missing component name in url"
-
-    component_request = ComponentRequest(request)
     component = UnicornView.create(
         component_id=component_request.id,
-        component_name=component_name,
+        component_name=component_request.name,
         request=request,
     )
     validate_all_fields = False
 
-    # Get a copy of the data passed in to determine what fields are updated later
-    original_data = component_request.data.copy()
+    # Get a deepcopy of the data passed in to determine what fields are updated later
+    original_data = copy.deepcopy(component_request.data)
 
     # Set component properties based on request data
     for (property_name, property_value) in component_request.data.items():
@@ -227,6 +291,7 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
     component.hydrate()
 
     is_reset_called = False
+    is_refresh_called = False
     return_data = None
     partials = []
 
@@ -319,16 +384,26 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
                     # Handle the refresh special action
                     component = UnicornView.create(
                         component_id=component_request.id,
-                        component_name=component_name,
-                        use_cache=True,
+                        component_name=component_request.name,
                         request=request,
                     )
+
+                    # Set component properties based on request data
+                    for (
+                        property_name,
+                        property_value,
+                    ) in component_request.data.items():
+                        _set_property_from_data(
+                            component, property_name, property_value
+                        )
+                    component.hydrate()
+
+                    is_refresh_called = True
                 elif method_name == "$reset":
                     # Handle the reset special action
                     component = UnicornView.create(
                         component_id=component_request.id,
-                        component_name=component_name,
-                        use_cache=False,
+                        component_name=component_request.name,
                         request=request,
                     )
 
@@ -356,17 +431,21 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
     # Re-load frontend context variables to deal with non-serializable properties
     component_request.data = orjson.loads(component.get_frontend_context_variables())
 
+    # Send back all available data for reset or refresh actions
+    updated_data = component_request.data
+
     if not is_reset_called:
-        if validate_all_fields:
-            component.validate()
-        else:
-            model_names_to_validate = []
+        if not is_refresh_called:
+            updated_data = {}
 
             for key, value in original_data.items():
                 if value != component_request.data.get(key):
-                    model_names_to_validate.append(key)
+                    updated_data[key] = component_request.data.get(key)
 
-            component.validate(model_names=model_names_to_validate)
+        if validate_all_fields:
+            component.validate()
+        else:
+            component.validate(model_names=list(updated_data.keys()))
 
     rendered_component = component.render()
     partial_doms = []
@@ -414,7 +493,7 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
 
     res = {
         "id": component_request.id,
-        "data": component_request.data,
+        "data": updated_data,
         "errors": component.errors,
         "checksum": generate_checksum(orjson.dumps(component_request.data)),
     }
@@ -442,33 +521,201 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
     parent_component = component.parent
 
     if parent_component:
-        parent_frontend_context_variables = (
+        parent_frontend_context_variables = loads(
             parent_component.get_frontend_context_variables()
         )
+        parent_checksum = generate_checksum(dumps(parent_frontend_context_variables))
 
-        parent_dom = parent_component.render()
+        parent = {
+            "id": parent_component.component_id,
+            "checksum": parent_checksum,
+        }
 
-        soup = BeautifulSoup(parent_dom, features="html.parser")
-        checksum = None
+        if not partial_doms:
+            parent_dom = parent_component.render()
 
-        # TODO: This doesn't create the same checksum for some reason
-        # checksum = orjson.dumps(parent_frontend_context_variables)
-
-        for element in soup.find_all():
-            if "unicorn:checksum" in element.attrs:
-                checksum = element["unicorn:checksum"]
-                break
-
-        res.update(
-            {
-                "parent": {
-                    "id": parent_component.component_id,
+            parent.update(
+                {
                     "dom": parent_dom,
-                    "checksum": checksum,
-                    "data": loads(parent_frontend_context_variables),
+                    "data": parent_frontend_context_variables,
                     "errors": parent_component.errors,
                 }
-            }
+            )
+
+        res.update({"parent": parent})
+
+    return res
+
+
+def _handle_component_request(
+    request: HttpRequest, component_request: ComponentRequest
+) -> Dict:
+    """
+    Process a `ComponentRequest` by adding it to the cache and then either:
+        - processing all of the component requests in the cache and returning the resulting value if
+            it is the first component request for that particular component name + component id combination
+        - return a `dict` saying that the request has been queued
+    
+    Args:
+        param request: HttpRequest for the function-based view.
+        param: component_request: Component request to process.
+    
+    Returns:
+        `dict` with the following structure:
+        {
+            "id": component_id,
+            "dom": html,  // re-rendered version of the component after actions in the payload are completed
+            "data": {},  // updated data after actions in the payload are completed
+            "errors": {},  // form validation errors
+            "return": {}, // optional return value from an executed action
+            "parent": {},  // optional representation of the parent component
+        }
+    """
+    # If serial isn't enabled or the wrong cache, just process the request like normal
+    if not get_serial_enabled():
+        return _process_component_request(request, component_request)
+
+    cache = caches[get_cache_alias()]
+
+    # Add the current request `ComponentRequest` to the cache
+    component_cache_key = f"{component_request.name}:{component_request.id}"
+    component_requests = cache.get(component_cache_key) or []
+    component_requests.append(component_request)
+
+    cache.set(
+        component_cache_key, component_requests, timeout=get_serial_timeout(),
+    )
+
+    if len(component_requests) > 1:
+        original_epoch = component_requests[0].epoch
+        return {
+            "queued": True,
+            "epoch": component_request.epoch,
+            "original_epoch": original_epoch,
+        }
+
+    return _handle_queued_component_requests(
+        request, component_request.name, component_cache_key
+    )
+
+
+def _handle_queued_component_requests(
+    request: HttpRequest, component_name: str, component_cache_key
+) -> Dict:
+    """
+    Process the current component requests that are stored in cache.
+    Also recursively checks for new requests that might have happened
+    while executing the first request, merges them together and returns
+    the correct appropriate data.
+
+    Args:
+        param request: HttpRequest for the view.
+        param: component_name: Name of the component, e.g. "hello-world".
+        param: component_cache_key: Cache key created from name of the component and the
+            component id which should be unique for any particular user's request lifecycle.
+    
+    Returns:
+        JSON with the following structure:
+        {
+            "id": component_id,
+            "dom": html,  // re-rendered version of the component after actions in the payload are completed
+            "data": {},  // updated data after actions in the payload are completed
+            "errors": {},  // form validation errors
+            "return": {}, // optional return value from an executed action
+            "parent": {},  // optional representation of the parent component
+        }
+    """
+    cache = caches[get_cache_alias()]
+
+    # Handle current request and any others in the cache by first sorting all of the current requests by ascending order
+    component_requests = cache.get(component_cache_key)
+
+    if not component_requests or not isinstance(component_requests, list):
+        raise UnicornViewError(f"No request found for {component_cache_key}")
+
+    component_requests = sorted(component_requests, key=lambda r: r.epoch)
+    first_component_request = component_requests[0]
+
+    # Can't store request on a `ComponentRequest` and cache it because `HttpRequest`
+    # isn't pickleable. Does it matter that a different request gets passed in then
+    # the original request that generated the `ComponentRequest`?
+    first_json_result = _process_component_request(request, first_component_request)
+
+    # Re-check for requests after the first request is processed
+    component_requests = cache.get(component_cache_key)
+
+    # Check that the request is in the cache before popping it off
+    if component_requests:
+        component_requests.pop(0)
+        cache.set(
+            component_cache_key, component_requests, timeout=get_serial_timeout(),
         )
 
-    return JsonResponse(res)
+    if component_requests:
+        # Create one new `component_request` from all of the queued requests that can be processed
+        merged_component_request = None
+
+        for additional_component_request in copy.deepcopy(component_requests):
+            if merged_component_request:
+                # Add new component request action queue to the merged component request
+                merged_component_request.action_queue.extend(
+                    additional_component_request.action_queue
+                )
+
+                # Originally, the thought was to merge the `additional_component_request.data` into
+                # the `merged_component_request.data`, but I can't figure out a way to do that in a sane
+                # manner. This means that for rapidly fired events that mutate `data`, that new
+                # `data` with be "thrown away".
+                # Relevant test: test_call_method_multiple.py::test_message_call_method_multiple_with_updated_data
+            else:
+                merged_component_request = additional_component_request
+
+                # Set new component request data from the first component request's resulting data
+                for key, val in first_json_result.get("data", {}).items():
+                    merged_component_request.data[key] = val
+
+            component_requests.pop(0)
+            cache.set(
+                component_cache_key, component_requests, timeout=get_serial_timeout(),
+            )
+
+        merged_json_result = _handle_component_request(
+            request, merged_component_request
+        )
+
+        return merged_json_result
+
+    return first_json_result
+
+
+@timed
+@handle_error
+@csrf_protect
+@require_POST
+def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
+    """
+    Endpoint that instantiates the component and does the correct action
+    (set an attribute or call a method) depending on the JSON payload in the body.
+
+    Args:
+        param request: HttpRequest for the function-based view.
+        param: component_name: Name of the component, e.g. "hello-world".
+    
+    Returns:
+        `JsonRequest` with the following structure in the body:
+        {
+            "id": component_id,
+            "dom": html,  // re-rendered version of the component after actions in the payload are completed
+            "data": {},  // updated data after actions in the payload are completed
+            "errors": {},  // form validation errors
+            "return": {}, // optional return value from an executed action
+            "parent": {},  // optional representation of the parent component
+        }
+    """
+
+    assert component_name, "Missing component name in url"
+
+    component_request = ComponentRequest(request, component_name)
+    json_result = _handle_component_request(request, component_request)
+
+    return JsonResponse(json_result)
